{- Copyright (c) 2015, Peter Ljung <ljung.peter@gmail.com>

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
USE OR PERFORMANCE OF THIS SOFTWARE. -}

module StockIo where

import Prelude as P
import TimeSeries as TS
import Data.Csv as CVS
import Data.Vector as V hiding ((++))
import Data.ByteString.Lazy.Char8 as B
import Network.HTTP
import Data.Time.Format as F

-- Stock time series represented as (Open,High,Low,Close,Volume)
type Share = TimeSeries (Float, Float, Float, Float, Float)
data Selector = Open | High | Low | Close | Volume

-- Parse CSV formatted share data:
-- Header: Date,Open,High,Low,Close,Volume
-- Data:   16-Jul-15,127.74,128.57,127.35,128.51,36222447
parseShare :: String -> Maybe Share
parseShare s | (P.null vs) = Nothing
             | otherwise         = Just (TS st TS.aDay as)
  where
    vs = case decode HasHeader $ pack s :: Either String (Vector (String, Float, Float, Float, Float, Float)) of
           Left _   -> []
           Right v  -> P.reverse $ V.toList v
    (ds,_,_,_,_,_) = P.head vs
    st = F.parseTimeOrError True defaultTimeLocale "%d-%h-%y" ds
    as = P.map (\(n,o,h,l,c,v) -> (o,h,l,c,v)) vs

-- Load share directly from google finance
-- curl "http://www.google.com/finance/historical?q=GOOG&output=csv"
loadShare :: String -> IO Share
loadShare share =  do
  let r = "http://www.google.com/finance/historical?q=" ++ share ++ "&output=csv"
  s <- simpleHTTP (getRequest r) >>= getResponseBody
  return $ case parseShare s of
             (Just ts) -> ts
             Nothing   -> TS.mkEmpty

-- Load share from file (same format as google finance)
-- File format: "./stocks/aapl.csv"
loadShareFile :: String -> IO Share
loadShareFile share = do
  let p = "./stocks/" ++ share ++ ".csv"
  s <- P.readFile p
  return $ case parseShare s of
             (Just ts) -> ts
             Nothing   -> TS.mkEmpty

-- Select one indicator from share
select :: Selector -> Share -> TimeSeries Float
select Open   = TS.map (\(o,_,_,_,_) -> o)
select High   = TS.map (\(_,h,_,_,_) -> h)
select Low    = TS.map (\(_,_,l,_,_) -> l)
select Close  = TS.map (\(_,_,_,c,_) -> c)
select Volume = TS.map (\(_,_,_,_,v) -> v)
